/**
 * Licensed to jclouds, Inc. (jclouds) under one or more
 * contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  jclouds licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.jclouds.abiquo.features;

import javax.inject.Named;
import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;

import org.jclouds.Fallbacks.NullOnNotFoundOr404;
import org.jclouds.abiquo.AbiquoFallbacks.FalseIfNotAvailable;
import org.jclouds.abiquo.AbiquoFallbacks.PropagateAbiquoExceptionOnNotFoundOr4xx;
import org.jclouds.abiquo.binders.AppendToPath;
import org.jclouds.abiquo.binders.BindToPath;
import org.jclouds.abiquo.binders.BindToXMLPayloadAndPath;
import org.jclouds.abiquo.binders.infrastructure.AppendMachineIdToPath;
import org.jclouds.abiquo.binders.infrastructure.AppendRemoteServiceTypeToPath;
import org.jclouds.abiquo.binders.infrastructure.BindSupportedDevicesLinkToPath;
import org.jclouds.abiquo.binders.infrastructure.ucs.BindLogicServerParameters;
import org.jclouds.abiquo.binders.infrastructure.ucs.BindOrganizationParameters;
import org.jclouds.abiquo.domain.infrastructure.options.DatacenterOptions;
import org.jclouds.abiquo.domain.infrastructure.options.IpmiOptions;
import org.jclouds.abiquo.domain.infrastructure.options.MachineOptions;
import org.jclouds.abiquo.domain.infrastructure.options.StoragePoolOptions;
import org.jclouds.abiquo.domain.network.options.IpOptions;
import org.jclouds.abiquo.domain.network.options.NetworkOptions;
import org.jclouds.abiquo.domain.options.search.FilterOptions;
import org.jclouds.abiquo.functions.infrastructure.ParseDatacenterId;
import org.jclouds.abiquo.http.filters.AbiquoAuthentication;
import org.jclouds.abiquo.http.filters.AppendApiVersionToMediaType;
import org.jclouds.abiquo.reference.annotations.EnterpriseEdition;
import org.jclouds.abiquo.rest.annotations.EndpointLink;
import org.jclouds.http.functions.ReturnStringIf2xx;
import org.jclouds.rest.annotations.BinderParam;
import org.jclouds.rest.annotations.Fallback;
import org.jclouds.rest.annotations.JAXBResponseParser;
import org.jclouds.rest.annotations.ParamParser;
import org.jclouds.rest.annotations.RequestFilters;
import org.jclouds.rest.annotations.ResponseParser;
import org.jclouds.rest.binders.BindToXMLPayload;

import com.abiquo.model.enumerator.HypervisorType;
import com.abiquo.model.enumerator.RemoteServiceType;
import com.abiquo.server.core.cloud.HypervisorTypesDto;
import com.abiquo.server.core.cloud.VirtualMachineWithNodeExtendedDto;
import com.abiquo.server.core.cloud.VirtualMachinesWithNodeExtendedDto;
import com.abiquo.server.core.enterprise.DatacentersLimitsDto;
import com.abiquo.server.core.enterprise.EnterpriseDto;
import com.abiquo.server.core.infrastructure.BladeLocatorLedDto;
import com.abiquo.server.core.infrastructure.DatacenterDto;
import com.abiquo.server.core.infrastructure.DatacentersDto;
import com.abiquo.server.core.infrastructure.FsmsDto;
import com.abiquo.server.core.infrastructure.LogicServerDto;
import com.abiquo.server.core.infrastructure.LogicServersDto;
import com.abiquo.server.core.infrastructure.MachineDto;
import com.abiquo.server.core.infrastructure.MachineIpmiStateDto;
import com.abiquo.server.core.infrastructure.MachineStateDto;
import com.abiquo.server.core.infrastructure.MachinesDto;
import com.abiquo.server.core.infrastructure.OrganizationDto;
import com.abiquo.server.core.infrastructure.OrganizationsDto;
import com.abiquo.server.core.infrastructure.RackDto;
import com.abiquo.server.core.infrastructure.RacksDto;
import com.abiquo.server.core.infrastructure.RemoteServiceDto;
import com.abiquo.server.core.infrastructure.RemoteServicesDto;
import com.abiquo.server.core.infrastructure.UcsRackDto;
import com.abiquo.server.core.infrastructure.UcsRacksDto;
import com.abiquo.server.core.infrastructure.network.ExternalIpDto;
import com.abiquo.server.core.infrastructure.network.ExternalIpsDto;
import com.abiquo.server.core.infrastructure.network.NetworkServiceTypeDto;
import com.abiquo.server.core.infrastructure.network.NetworkServiceTypesDto;
import com.abiquo.server.core.infrastructure.network.PublicIpDto;
import com.abiquo.server.core.infrastructure.network.PublicIpsDto;
import com.abiquo.server.core.infrastructure.network.UnmanagedIpDto;
import com.abiquo.server.core.infrastructure.network.UnmanagedIpsDto;
import com.abiquo.server.core.infrastructure.network.VLANNetworkDto;
import com.abiquo.server.core.infrastructure.network.VLANNetworksDto;
import com.abiquo.server.core.infrastructure.network.VlanTagAvailabilityDto;
import com.abiquo.server.core.infrastructure.storage.StorageDeviceDto;
import com.abiquo.server.core.infrastructure.storage.StorageDevicesDto;
import com.abiquo.server.core.infrastructure.storage.StorageDevicesMetadataDto;
import com.abiquo.server.core.infrastructure.storage.StoragePoolDto;
import com.abiquo.server.core.infrastructure.storage.StoragePoolsDto;
import com.abiquo.server.core.infrastructure.storage.TierDto;
import com.abiquo.server.core.infrastructure.storage.TiersDto;
import com.google.common.util.concurrent.ListenableFuture;

/**
 * Provides asynchronous access to Abiquo Infrastructure API.
 * 
 * @see API: <a href="http://community.abiquo.com/display/ABI20/API+Reference">
 *      http://community.abiquo.com/display/ABI20/API+Reference</a>
 * @see InfrastructureApi
 * @author Ignasi Barrera
 * @author Francesc Montserrat
 */
@RequestFilters({ AbiquoAuthentication.class, AppendApiVersionToMediaType.class })
@Path("/admin")
public interface InfrastructureAsyncApi {
   /*********************** Datacenter ***********************/

   /**
    * @see InfrastructureApi#listDatacenters()
    */
   @Named("datacenter:list")
   @GET
   @Path("/datacenters")
   @Consumes(DatacentersDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<DatacentersDto> listDatacenters();

   /**
    * @see InfrastructureApi#createDatacenter(DatacenterDto)
    */
   @Named("datacenter:create")
   @POST
   @Path("/datacenters")
   @Produces(DatacenterDto.BASE_MEDIA_TYPE)
   @Consumes(DatacenterDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<DatacenterDto> createDatacenter(@BinderParam(BindToXMLPayload.class) DatacenterDto datacenter);

   /**
    * @see InfrastructureApi#getDatacenter(Integer)
    */
   @Named("datacenter:get")
   @GET
   @Path("/datacenters/{datacenter}")
   @Consumes(DatacenterDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   @Fallback(NullOnNotFoundOr404.class)
   ListenableFuture<DatacenterDto> getDatacenter(@PathParam("datacenter") Integer datacenterId);

   /**
    * @see InfrastructureApi#updateDatacenter(DatacenterDto)
    */
   @Named("datacenter:update")
   @PUT
   @Produces(DatacenterDto.BASE_MEDIA_TYPE)
   @Consumes(DatacenterDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<DatacenterDto> updateDatacenter(
         @EndpointLink("edit") @BinderParam(BindToXMLPayloadAndPath.class) DatacenterDto datacenter);

   /**
    * @see InfrastructureApi#deleteDatacenter(DatacenterDto)
    */
   @Named("datacenter:delete")
   @DELETE
   ListenableFuture<Void> deleteDatacenter(@EndpointLink("edit") @BinderParam(BindToPath.class) DatacenterDto datacenter);

   /**
    * @see InfrastructureApi#discoverSingleMachine(DatacenterDto, String,
    *      HypervisorType, String, String)
    */
   @Named("machine:discover")
   @GET
   @Consumes(MachineDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   @Fallback(PropagateAbiquoExceptionOnNotFoundOr4xx.class)
   ListenableFuture<MachineDto> discoverSingleMachine(
         @EndpointLink("discoversingle") @BinderParam(BindToPath.class) DatacenterDto datacenter,
         @QueryParam("ip") String ip, @QueryParam("hypervisor") HypervisorType hypervisorType,
         @QueryParam("user") String user, @QueryParam("password") String password);

   /**
    * @see InfrastructureApi#discoverSingleMachine(DatacenterDto, String,
    *      HypervisorType, String, String, MachineOptions)
    */
   @Named("machine:discover")
   @GET
   @Consumes(MachineDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   @Fallback(PropagateAbiquoExceptionOnNotFoundOr4xx.class)
   ListenableFuture<MachineDto> discoverSingleMachine(
         @EndpointLink("discoversingle") @BinderParam(BindToPath.class) DatacenterDto datacenter,
         @QueryParam("ip") String ip, @QueryParam("hypervisor") HypervisorType hypervisorType,
         @QueryParam("user") String user, @QueryParam("password") String password, MachineOptions options);

   /**
    * @see InfrastructureApi#discoverMultipleMachines(DatacenterDto, String,
    *      String, HypervisorType, String, String)
    */
   @Named("machine:discover")
   @GET
   @Consumes(MachinesDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   @Fallback(PropagateAbiquoExceptionOnNotFoundOr4xx.class)
   ListenableFuture<MachineDto> discoverMultipleMachines(
         @EndpointLink("discovermultiple") @BinderParam(BindToPath.class) DatacenterDto datacenter,
         @QueryParam("ipFrom") String ipFrom, @QueryParam("ipTo") String ipTo,
         @QueryParam("hypervisor") HypervisorType hypervisorType, @QueryParam("user") String user,
         @QueryParam("password") String password);

   /**
    * @see InfrastructureApi#discoverMultipleMachines(DatacenterDto, String,
    *      String, HypervisorType, String, String, MachineOptions)
    */
   @Named("machine:discover")
   @GET
   @Consumes(MachinesDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   @Fallback(PropagateAbiquoExceptionOnNotFoundOr4xx.class)
   ListenableFuture<MachineDto> discoverMultipleMachines(
         @EndpointLink("discovermultiple") @BinderParam(BindToPath.class) DatacenterDto datacenter,
         @QueryParam("ipFrom") String ipFrom, @QueryParam("ipTo") String ipTo,
         @QueryParam("hypervisor") HypervisorType hypervisorType, @QueryParam("user") String user,
         @QueryParam("password") String password, MachineOptions options);

   /**
    * @see InfrastructureApi#listLimits(DatacenterDto)
    */
   @Named("limit:list")
   @GET
   @Consumes(DatacentersLimitsDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<DatacentersLimitsDto> listLimits(
         @EndpointLink("getLimits") @BinderParam(BindToPath.class) DatacenterDto datacenter);

   /**
    * @see InfrastructureApi#checkMachineState(DatacenterDto, String, String,
    *      HypervisorType, String, String)
    */
   @Named("machine:checkstate")
   @GET
   @Consumes(MachineStateDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   @Fallback(PropagateAbiquoExceptionOnNotFoundOr4xx.class)
   ListenableFuture<MachineStateDto> checkMachineState(
         @EndpointLink("checkmachinestate") @BinderParam(BindToPath.class) DatacenterDto datacenter,
         @QueryParam("ip") String ip, @QueryParam("hypervisor") HypervisorType hypervisorType,
         @QueryParam("user") String user, @QueryParam("password") String password);

   /**
    * @see InfrastructureApi#checkMachineState(DatacenterDto, String, String,
    *      HypervisorType, String, String, MachineOptions)
    */
   @Named("machine:checkstate")
   @GET
   @Consumes(MachineStateDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   @Fallback(PropagateAbiquoExceptionOnNotFoundOr4xx.class)
   ListenableFuture<MachineStateDto> checkMachineState(
         @EndpointLink("checkmachinestate") @BinderParam(BindToPath.class) DatacenterDto datacenter,
         @QueryParam("ip") String ip, @QueryParam("hypervisor") HypervisorType hypervisorType,
         @QueryParam("user") String user, @QueryParam("password") String password, MachineOptions options);

   /**
    * @see InfrastructureApi#checkMachineIpmiState(DatacenterDto, String,
    *      String, String)
    */
   @Named("machine:checkipmi")
   @GET
   @Consumes(MachineIpmiStateDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   @Fallback(PropagateAbiquoExceptionOnNotFoundOr4xx.class)
   ListenableFuture<MachineIpmiStateDto> checkMachineIpmiState(
         @EndpointLink("checkmachineipmistate") @BinderParam(BindToPath.class) DatacenterDto datacenter,
         @QueryParam("ip") String ip, @QueryParam("user") String user, @QueryParam("password") String password);

   /**
    * @see InfrastructureApi#checkMachineIpmiState(DatacenterDto, String,
    *      String, String, IpmiOptions)
    */
   @Named("machine:checkipmi")
   @GET
   @Consumes(MachineIpmiStateDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   @Fallback(PropagateAbiquoExceptionOnNotFoundOr4xx.class)
   ListenableFuture<MachineIpmiStateDto> checkMachineIpmiState(
         @EndpointLink("checkmachineipmistate") @BinderParam(BindToPath.class) DatacenterDto datacenter,
         @QueryParam("ip") String ip, @QueryParam("user") String user, @QueryParam("password") String password,
         IpmiOptions options);

   /*********************** Hypervisor ***********************/
   /**
    * @see InfrastructureApi#getHypervisorTypeFromMachine(DatacenterDto,
    *      DatacenterOptions)
    */
   @Named("hypervisortype:getfrommachine")
   @GET
   @Consumes(MediaType.TEXT_PLAIN)
   @ResponseParser(ReturnStringIf2xx.class)
   ListenableFuture<String> getHypervisorTypeFromMachine(
         @EndpointLink("hypervisor") @BinderParam(BindToPath.class) DatacenterDto datacenter, DatacenterOptions options);

   /**
    * @see InfrastructureApi#getHypervisorTypes(DatacenterDto)
    */
   @Named("hypervisortype:list")
   @GET
   @Consumes(HypervisorTypesDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<HypervisorTypesDto> getHypervisorTypes(
         @EndpointLink("hypervisors") @BinderParam(BindToPath.class) DatacenterDto datacenter);

   /*********************** Unmanaged Rack ***********************/

   /**
    * @see InfrastructureApi#listRacks(DatacenterDto)
    */
   @Named("rack:list")
   @GET
   @Consumes(RacksDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<RacksDto> listRacks(@EndpointLink("racks") @BinderParam(BindToPath.class) DatacenterDto datacenter);

   /**
    * @see InfrastructureApi#createRack(DatacenterDto, RackDto)
    */
   @Named("rack:create")
   @POST
   @Produces(RackDto.BASE_MEDIA_TYPE)
   @Consumes(RackDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<RackDto> createRack(@EndpointLink("racks") @BinderParam(BindToPath.class) DatacenterDto datacenter,
         @BinderParam(BindToXMLPayload.class) RackDto rack);

   /**
    * @see InfrastructureApi#getRack(DatacenterDto, Integer)
    */
   @Named("rack:get")
   @GET
   @Consumes(RackDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   @Fallback(NullOnNotFoundOr404.class)
   ListenableFuture<RackDto> getRack(@EndpointLink("racks") @BinderParam(BindToPath.class) DatacenterDto datacenter,
         @BinderParam(AppendToPath.class) Integer rackId);

   /**
    * @see InfrastructureApi#updateRack(RackDto)
    */
   @Named("rack:update")
   @PUT
   @Consumes(RackDto.BASE_MEDIA_TYPE)
   @Produces(RackDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<RackDto> updateRack(@EndpointLink("edit") @BinderParam(BindToXMLPayloadAndPath.class) RackDto rack);

   /**
    * @see InfrastructureApi#deleteRack(RackDto)
    */
   @Named("rack:delete")
   @DELETE
   ListenableFuture<Void> deleteRack(@EndpointLink("edit") @BinderParam(BindToPath.class) RackDto rack);

   /*********************** Managed Rack ***********************/

   /**
    * @see InfrastructureApi#listManagedRacks(DatacenterDto)
    */
   @Named("ucs:listracks")
   @EnterpriseEdition
   @GET
   @Consumes(UcsRacksDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<UcsRacksDto> listManagedRacks(
         @EndpointLink("racks") @BinderParam(BindToPath.class) DatacenterDto datacenter);

   /**
    * @see InfrastructureApi#createManagedRack(DatacenterDto, UcsRackDto)
    */
   @Named("ucs:createrack")
   @EnterpriseEdition
   @POST
   @Produces(UcsRackDto.BASE_MEDIA_TYPE)
   @Consumes(UcsRackDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<UcsRackDto> createManagedRack(
         @EndpointLink("racks") @BinderParam(BindToPath.class) DatacenterDto datacenter,
         @BinderParam(BindToXMLPayload.class) UcsRackDto rack);

   /**
    * @see InfrastructureApi#getManagedRack(DatacenterDto, Integer)
    */
   @Named("ucs:getrack")
   @EnterpriseEdition
   @GET
   @Consumes(UcsRackDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   @Fallback(NullOnNotFoundOr404.class)
   ListenableFuture<UcsRackDto> getManagedRack(
         @EndpointLink("racks") @BinderParam(BindToPath.class) DatacenterDto datacenter,
         @BinderParam(AppendToPath.class) Integer rackId);

   /**
    * @see InfrastructureApi#updateManagedRack(UcsRackDto)
    */
   @Named("ucs:updaterack")
   @EnterpriseEdition
   @PUT
   @Consumes(UcsRackDto.BASE_MEDIA_TYPE)
   @Produces(UcsRackDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<UcsRackDto> updateManagedRack(
         @EndpointLink("edit") @BinderParam(BindToXMLPayloadAndPath.class) UcsRackDto rack);

   /**
    * @see InfrastructureApi#listServiceProfiles(UcsRackDto)
    */
   @Named("ucs:listserviceprofiles")
   @EnterpriseEdition
   @GET
   @Consumes(LogicServersDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<LogicServersDto> listServiceProfiles(
         @EndpointLink("logicservers") @BinderParam(BindToPath.class) UcsRackDto rack);

   /**
    * @see InfrastructureApi#listServiceProfiles(UcsRackDto, QueryOptions)
    */
   @Named("ucs:listserviceprofiles")
   @EnterpriseEdition
   @GET
   @Consumes(LogicServersDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<LogicServersDto> listServiceProfiles(
         @EndpointLink("logicservers") @BinderParam(BindToPath.class) UcsRackDto rack, FilterOptions options);

   /**
    * @see InfrastructureApi#listServiceProfileTemplates(UcsRackDto)
    */
   @Named("ucs:listserviceprofiletemplates")
   @EnterpriseEdition
   @GET
   @Consumes(LogicServersDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<LogicServersDto> listServiceProfileTemplates(
         @EndpointLink("ls-templates") @BinderParam(BindToPath.class) UcsRackDto rack);

   /**
    * @see InfrastructureApi#listServiceProfileTemplates(UcsRackDto,
    *      LogicServerOptions)
    */
   @Named("ucs:listserviceproviletemplates")
   @EnterpriseEdition
   @GET
   @Consumes(LogicServersDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<LogicServersDto> listServiceProfileTemplates(
         @EndpointLink("ls-templates") @BinderParam(BindToPath.class) UcsRackDto rack, FilterOptions options);

   /**
    * @see InfrastructureApi#listOrganizations(UcsRackDto)
    */
   @Named("ucs:listorganizations")
   @EnterpriseEdition
   @GET
   @Consumes(OrganizationsDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<OrganizationsDto> listOrganizations(
         @EndpointLink("organizations") @BinderParam(BindToPath.class) UcsRackDto rack);

   /**
    * @see InfrastructureApi#listOrganizations(UcsRackDto, OrganizationOptions)
    */
   @Named("ucs:listorganizations")
   @EnterpriseEdition
   @GET
   @Consumes(OrganizationsDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<OrganizationsDto> listOrganizations(
         @EndpointLink("organizations") @BinderParam(BindToPath.class) UcsRackDto rack, FilterOptions options);

   /**
    * @see InfrastructureApi#cloneLogicServer(UcsRackDto, LogicServerDto,
    *      OrganizationDto, String)
    */
   @Named("ucs:clonelogicserver")
   @EnterpriseEdition
   @POST
   ListenableFuture<Void> cloneLogicServer(@EndpointLink("ls-clone") @BinderParam(BindToPath.class) UcsRackDto rack,
         @BinderParam(BindLogicServerParameters.class) LogicServerDto logicServer,
         @BinderParam(BindOrganizationParameters.class) OrganizationDto organization,
         @QueryParam("newName") String newName);

   /**
    * @see InfrastructureApi#associateLogicServer(UcsRackDto, LogicServerDto,
    *      OrganizationDto, String)
    */
   @Named("ucs:associatelogicserver")
   @EnterpriseEdition
   @POST
   ListenableFuture<Void> associateLogicServer(
         @EndpointLink("ls-associate") @BinderParam(BindToPath.class) UcsRackDto rack,
         @BinderParam(BindLogicServerParameters.class) LogicServerDto logicServer,
         @BinderParam(BindOrganizationParameters.class) OrganizationDto organization,
         @QueryParam("bladeDn") String bladeName);

   /**
    * @see InfrastructureApi#associateTemplate(UcsRackDto, LogicServerDto,
    *      OrganizationDto, String, String)
    */
   @Named("ucs:associatetemplate")
   @EnterpriseEdition
   @POST
   ListenableFuture<Void> associateTemplate(
         @EndpointLink("ls-associatetemplate") @BinderParam(BindToPath.class) UcsRackDto rack,
         @BinderParam(BindLogicServerParameters.class) LogicServerDto logicServer,
         @BinderParam(BindOrganizationParameters.class) OrganizationDto organization,
         @QueryParam("newName") String newName, @QueryParam("bladeDn") String bladeName);

   /**
    * @see InfrastructureApi#cloneAndAssociateLogicServer(UcsRackDto,
    *      LogicServerDto, OrganizationDto, String, String)
    */
   @Named("ucs:cloneandassociatelogicserver")
   @EnterpriseEdition
   @POST
   ListenableFuture<Void> cloneAndAssociateLogicServer(
         @EndpointLink("ls-associateclone") @BinderParam(BindToPath.class) UcsRackDto rack,
         @BinderParam(BindLogicServerParameters.class) LogicServerDto logicServer,
         @BinderParam(BindOrganizationParameters.class) OrganizationDto organization,
         @QueryParam("newName") String newName, @QueryParam("bladeDn") String bladeName);

   /**
    * @see InfrastructureApi#dissociateLogicServer(UcsRackDto, LogicServerDto)
    */
   @Named("ucs:dissociatelogicserver")
   @EnterpriseEdition
   @POST
   ListenableFuture<Void> dissociateLogicServer(
         @EndpointLink("ls-dissociate") @BinderParam(BindToPath.class) UcsRackDto rack,
         @BinderParam(BindLogicServerParameters.class) LogicServerDto logicServer);

   /**
    * @see InfrastructureApi#deleteLogicServer(UcsRackDto, LogicServerDto)
    */
   @Named("ucs:deletelogicserver")
   @EnterpriseEdition
   @POST
   ListenableFuture<Void> deleteLogicServer(@EndpointLink("ls-delete") @BinderParam(BindToPath.class) UcsRackDto rack,
         @BinderParam(BindLogicServerParameters.class) LogicServerDto logicServer);

   /**
    * @see InfrastructureApi#listFsms(UcsRackDto, String)
    */
   @Named("ucs:listfsms")
   @EnterpriseEdition
   @GET
   @Consumes(FsmsDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<FsmsDto> listFsms(@EndpointLink("fsm") @BinderParam(BindToPath.class) UcsRackDto rack,
         @QueryParam("dn") String dn);

   /*********************** Remote Service ***********************/

   /**
    * @see InfrastructureApi#listRemoteServices(DatacenterDto)
    */
   @Named("rs:list")
   @GET
   @Consumes(RemoteServicesDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<RemoteServicesDto> listRemoteServices(
         @EndpointLink("remoteservices") @BinderParam(BindToPath.class) DatacenterDto datacenter);

   /**
    * @see InfrastructureApi#createRemoteService(DatacenterDto,
    *      RemoteServiceDto)
    */
   @Named("rs:create")
   @POST
   @Produces(RemoteServiceDto.BASE_MEDIA_TYPE)
   @Consumes(RemoteServiceDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<RemoteServiceDto> createRemoteService(
         @EndpointLink("remoteservices") @BinderParam(BindToPath.class) DatacenterDto datacenter,
         @BinderParam(BindToXMLPayload.class) RemoteServiceDto remoteService);

   /**
    * @see InfrastructureApi#getRemoteService(DatacenterDto, RemoteServiceType)
    */
   @Named("rs:get")
   @GET
   @Consumes(RemoteServiceDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   @Fallback(NullOnNotFoundOr404.class)
   ListenableFuture<RemoteServiceDto> getRemoteService(
         @EndpointLink("remoteservices") @BinderParam(BindToPath.class) final DatacenterDto datacenter,
         @BinderParam(AppendRemoteServiceTypeToPath.class) final RemoteServiceType remoteServiceType);

   /**
    * @see InfrastructureApi#updateRemoteService(RemoteServiceDto)
    */
   @Named("rs:update")
   @PUT
   @Consumes(RemoteServiceDto.BASE_MEDIA_TYPE)
   @Produces(RemoteServiceDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<RemoteServiceDto> updateRemoteService(
         @EndpointLink("edit") @BinderParam(BindToXMLPayloadAndPath.class) RemoteServiceDto remoteService);

   /**
    * @see InfrastructureApi#deleteRemoteService(RemoteServiceDto)
    */
   @Named("rs:delete")
   @DELETE
   ListenableFuture<Void> deleteRemoteService(
         @EndpointLink("edit") @BinderParam(BindToPath.class) RemoteServiceDto remoteService);

   /**
    * @see InfrastructureApi#isAvailable(RemoteServiceDto)
    */
   @Named("rs:available")
   @GET
   @Fallback(FalseIfNotAvailable.class)
   ListenableFuture<Boolean> isAvailable(
         @EndpointLink("check") @BinderParam(BindToPath.class) RemoteServiceDto remoteService);

   /*********************** Machine ***********************/

   /**
    * @see InfrastructureApi#listMachines(RackDto)
    */
   @Named("machine:list")
   @GET
   @Consumes(MachinesDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<MachinesDto> listMachines(@EndpointLink("machines") @BinderParam(BindToPath.class) RackDto rack);

   /**
    * @see InfrastructureApi#createMachine(RackDto, MachineDto)
    */
   @Named("machine:create")
   @POST
   @Produces(MachineDto.BASE_MEDIA_TYPE)
   @Consumes(MachineDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<MachineDto> createMachine(@EndpointLink("machines") @BinderParam(BindToPath.class) RackDto rack,
         @BinderParam(BindToXMLPayload.class) MachineDto machine);

   /**
    * @see InfrastructureApi#getMachine(RackDto, Integer)
    */
   @Named("machine:get")
   @GET
   @Consumes(MachineDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   @Fallback(NullOnNotFoundOr404.class)
   ListenableFuture<MachineDto> getMachine(@EndpointLink("machines") @BinderParam(BindToPath.class) final RackDto rack,
         @BinderParam(AppendToPath.class) Integer machineId);

   /**
    * @see InfrastructureApi#checkMachineState(MachineDto)
    */
   @Named("machine:checkstate")
   @GET
   @Consumes(MachineStateDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<MachineStateDto> checkMachineState(
         @EndpointLink("checkstate") @BinderParam(BindToPath.class) final MachineDto machine,
         @QueryParam("sync") boolean sync);

   /**
    * @see InfrastructureApi#checkMachineIpmiState(MachineDto)
    */
   @Named("machine:checkipmi")
   @GET
   @Consumes(MachineIpmiStateDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<MachineIpmiStateDto> checkMachineIpmiState(
         @EndpointLink("checkipmistate") @BinderParam(BindToPath.class) final MachineDto machine);

   /**
    * @see InfrastructureApi#updateMachine(MachineDto)
    */
   @Named("machine:update")
   @PUT
   @Produces(MachineDto.BASE_MEDIA_TYPE)
   @Consumes(MachineDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<MachineDto> updateMachine(
         @EndpointLink("edit") @BinderParam(BindToXMLPayloadAndPath.class) MachineDto machine);

   /**
    * @see InfrastructureApi#deleteMachine(MachineDto)
    */
   @Named("machine:delete")
   @DELETE
   ListenableFuture<Void> deleteMachine(@EndpointLink("edit") @BinderParam(BindToPath.class) MachineDto machine);

   /**
    * @see InfrastructureApi#reserveMachine(EnterpriseDto, MachineDto)
    */
   @Named("machine:reserve")
   @POST
   @Consumes(MachineDto.BASE_MEDIA_TYPE)
   @Produces(MachineDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<MachineDto> reserveMachine(
         @EndpointLink("reservedmachines") @BinderParam(BindToPath.class) EnterpriseDto enterprise,
         @BinderParam(BindToXMLPayload.class) MachineDto machine);

   /**
    * @see InfrastructureApi#cancelReservation(EnterpriseDto, MachineDto)
    */
   @Named("machine:cancelreservation")
   @DELETE
   ListenableFuture<Void> cancelReservation(
         @EndpointLink("reservedmachines") @BinderParam(BindToPath.class) EnterpriseDto enterprise,
         @BinderParam(AppendMachineIdToPath.class) MachineDto machine);

   /*********************** Blade ***********************/

   /**
    * @see InfrastructureApi#powerOff(MachineDto)
    */
   @Named("machine:poweroff")
   @EnterpriseEdition
   @PUT
   ListenableFuture<Void> powerOff(@EndpointLink("poweroff") @BinderParam(BindToPath.class) MachineDto machine);

   /**
    * @see InfrastructureApi#powerOn(MachineDto)
    */
   @Named("machine:poweron")
   @EnterpriseEdition
   @PUT
   ListenableFuture<Void> powerOn(@EndpointLink("poweron") @BinderParam(BindToPath.class) MachineDto machine);

   /**
    * @see InfrastructureApi#getLogicServer(MachineDto)
    */
   @Named("machine:getlogicserver")
   @EnterpriseEdition
   @GET
   @Consumes(LogicServerDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<LogicServerDto> getLogicServer(
         @EndpointLink("logicserver") @BinderParam(BindToPath.class) MachineDto machine);

   /**
    * @see InfrastructureApi#ledOn(MachineDto)
    */
   @Named("machine:ledon")
   @EnterpriseEdition
   @POST
   ListenableFuture<Void> ledOn(@EndpointLink("ledon") @BinderParam(BindToPath.class) MachineDto machine);

   /**
    * @see InfrastructureApi#ledOff(MachineDto)
    */
   @Named("machine:ledoff")
   @EnterpriseEdition
   @POST
   ListenableFuture<Void> ledOff(@EndpointLink("ledoff") @BinderParam(BindToPath.class) MachineDto machine);

   /**
    * @see InfrastructureApi#getLedLocator(MachineDto)
    */
   @Named("machine:getlocatorled")
   @EnterpriseEdition
   @GET
   @Consumes(BladeLocatorLedDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<BladeLocatorLedDto> getLocatorLed(
         @EndpointLink("led") @BinderParam(BindToPath.class) MachineDto machine);

   /**
    * @see InfrastructureApi#listVirtualMachinesByMachine(MachineDto)
    */
   @Named("machine:listvms")
   @GET
   @Consumes(VirtualMachinesWithNodeExtendedDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<VirtualMachinesWithNodeExtendedDto> listVirtualMachinesByMachine(
         @EndpointLink("virtualmachines") @BinderParam(BindToPath.class) MachineDto machine, MachineOptions options);

   /**
    * @see InfrastructureApi#getVirtualMachine(MachineDto, Integer)
    */
   @Named("machine:getvm")
   @GET
   @Fallback(NullOnNotFoundOr404.class)
   @Consumes(VirtualMachineWithNodeExtendedDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<VirtualMachineWithNodeExtendedDto> getVirtualMachine(
         @EndpointLink("virtualmachines") @BinderParam(BindToPath.class) MachineDto machine,
         @BinderParam(AppendToPath.class) Integer virtualMachineId);

   /*********************** Storage Device ***********************/

   /**
    * @see InfrastructureApi#listStorageDevices(DatacenterDto)
    */
   @Named("storagedevice:list")
   @EnterpriseEdition
   @GET
   @Consumes(StorageDevicesDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<StorageDevicesDto> listStorageDevices(
         @EndpointLink("devices") @BinderParam(BindToPath.class) DatacenterDto datacenter);

   /**
    * @see InfrastructureApi#listSupportedStorageDevices(DatacenterDto)
    */
   @Named("storagedevice:listsupported")
   @EnterpriseEdition
   @GET
   @Consumes(StorageDevicesMetadataDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<StorageDevicesMetadataDto> listSupportedStorageDevices(
         @EndpointLink("devices") @BinderParam(BindSupportedDevicesLinkToPath.class) DatacenterDto datacenter);

   /**
    * @see InfrastructureApi#getStorageDevice(DatacenterDto, Integer)
    */
   @Named("storagedevice:get")
   @EnterpriseEdition
   @GET
   @Consumes(StorageDeviceDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   @Fallback(NullOnNotFoundOr404.class)
   ListenableFuture<StorageDeviceDto> getStorageDevice(
         @EndpointLink("devices") @BinderParam(BindToPath.class) DatacenterDto datacenter,
         @BinderParam(AppendToPath.class) Integer storageDeviceId);

   /**
    * @see InfrastructureApi#createStorageDevice(DatacenterDto,
    *      StorageDeviceDto)
    */
   @Named("storagedevice:create")
   @EnterpriseEdition
   @POST
   @Produces(StorageDeviceDto.BASE_MEDIA_TYPE)
   @Consumes(StorageDeviceDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<StorageDeviceDto> createStorageDevice(
         @EndpointLink("devices") @BinderParam(BindToPath.class) DatacenterDto datacenter,
         @BinderParam(BindToXMLPayload.class) StorageDeviceDto storageDevice);

   /**
    * @see InfrastructureApi#deleteStorageDevice(StorageDeviceDto)
    */
   @Named("storagedevice:delete")
   @EnterpriseEdition
   @DELETE
   ListenableFuture<Void> deleteStorageDevice(
         @EndpointLink("edit") @BinderParam(BindToPath.class) StorageDeviceDto storageDevice);

   /**
    * @see InfrastructureApi#updateStorageDevice(StorageDeviceDto)
    */
   @Named("storagedevice:update")
   @EnterpriseEdition
   @PUT
   @Produces(StorageDeviceDto.BASE_MEDIA_TYPE)
   @Consumes(StorageDeviceDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<StorageDeviceDto> updateStorageDevice(
         @EndpointLink("edit") @BinderParam(BindToXMLPayloadAndPath.class) StorageDeviceDto storageDevice);

   /*********************** Tier ***********************/

   /**
    * @see InfrastructureApi#listTiers(DatacenterDto)
    */
   @Named("tier:list")
   @EnterpriseEdition
   @GET
   @Consumes(TiersDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<TiersDto> listTiers(@EndpointLink("tiers") @BinderParam(BindToPath.class) DatacenterDto datacenter);

   /**
    * @see InfrastructureApi#updateTier(TierDto)
    */
   @Named("tier:update")
   @EnterpriseEdition
   @PUT
   @Produces(TierDto.BASE_MEDIA_TYPE)
   @Consumes(TierDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<TierDto> updateTier(@EndpointLink("edit") @BinderParam(BindToXMLPayloadAndPath.class) TierDto tier);

   /**
    * @see InfrastructureApi#getTier(DatacenterDto, Integer)
    */
   @Named("tier:get")
   @EnterpriseEdition
   @GET
   @Consumes(TierDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   @Fallback(NullOnNotFoundOr404.class)
   ListenableFuture<TierDto> getTier(@EndpointLink("tiers") @BinderParam(BindToPath.class) DatacenterDto datacenter,
         @BinderParam(AppendToPath.class) Integer tierId);

   /*********************** Storage Pool ***********************/

   /**
    * @see InfrastructureApi#listStoragePools(StorageDeviceDto,
    *      StoragePoolOptions)
    */
   @Named("storagepool:list")
   @EnterpriseEdition
   @GET
   @Consumes(StoragePoolsDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<StoragePoolsDto> listStoragePools(
         @EndpointLink("pools") @BinderParam(BindToPath.class) StorageDeviceDto storageDevice,
         StoragePoolOptions options);

   /**
    * @see InfrastructureApi#listStoragePools(TierDto)
    */
   @Named("storagepool:list")
   @EnterpriseEdition
   @GET
   @Consumes(StoragePoolsDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<StoragePoolsDto> listStoragePools(@EndpointLink("pools") @BinderParam(BindToPath.class) TierDto tier);

   /**
    * @see InfrastructureApi#createStoragePool(StorageDeviceDto, StoragePoolDto)
    */
   @Named("storagepool:create")
   @EnterpriseEdition
   @POST
   @Consumes(StoragePoolDto.BASE_MEDIA_TYPE)
   @Produces(StoragePoolDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<StoragePoolDto> createStoragePool(
         @EndpointLink("pools") @BinderParam(BindToPath.class) StorageDeviceDto storageDevice,
         @BinderParam(BindToXMLPayload.class) StoragePoolDto storagePool);

   /**
    * @see InfrastructureApi#updateStoragePool(StoragePoolDto)
    */
   @Named("storagepool:update")
   @EnterpriseEdition
   @PUT
   // For the most strangest reason in world, compiler does not accept
   // constants StoragePoolDto.BASE_MEDIA_TYPE for this method.
   @Consumes("application/vnd.abiquo.storagepool+xml")
   @Produces("application/vnd.abiquo.storagepool+xml")
   @JAXBResponseParser
   ListenableFuture<StoragePoolDto> updateStoragePool(
         @EndpointLink("edit") @BinderParam(BindToXMLPayloadAndPath.class) StoragePoolDto StoragePoolDto);

   /**
    * @see InfrastructureApi#deleteStoragePool(StoragePoolDto)
    */
   @Named("storagepool:delete")
   @EnterpriseEdition
   @DELETE
   ListenableFuture<Void> deleteStoragePool(
         @EndpointLink("edit") @BinderParam(BindToPath.class) StoragePoolDto storagePool);

   /**
    * @see InfrastructureApi#getStoragePool(StorageDeviceDto, String)
    */
   @Named("storagepool:get")
   @EnterpriseEdition
   @GET
   @Consumes(StoragePoolDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   @Fallback(NullOnNotFoundOr404.class)
   ListenableFuture<StoragePoolDto> getStoragePool(
         @EndpointLink("pools") @BinderParam(BindToPath.class) final StorageDeviceDto storageDevice,
         @BinderParam(AppendToPath.class) final String storagePoolId);

   /**
    * @see InfrastructureApi#refreshStoragePool(StoragePoolDto,
    *      StoragePoolOptions)
    */
   @Named("storagepool:refresh")
   @EnterpriseEdition
   @GET
   @Consumes(StoragePoolDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<StoragePoolDto> refreshStoragePool(
         @EndpointLink("edit") @BinderParam(BindToPath.class) StoragePoolDto storagePool, StoragePoolOptions options);

   /*********************** Network ***********************/

   /**
    * @see InfrastructureApi#listNetworks(DatacenterDto)
    */
   @Named("network:list")
   @EnterpriseEdition
   @GET
   @Consumes(VLANNetworksDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<VLANNetworksDto> listNetworks(
         @EndpointLink("network") @BinderParam(BindToPath.class) DatacenterDto datacenter);

   /**
    * @see InfrastructureApi#listNetwork(DatacenterDto, NetworkOptions)
    */
   @Named("network:list")
   @EnterpriseEdition
   @GET
   @Consumes(VLANNetworksDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<VLANNetworksDto> listNetworks(
         @EndpointLink("network") @BinderParam(BindToPath.class) DatacenterDto datacenter, NetworkOptions options);

   /**
    * @see InfrastructureApi#getNetwork(DatacenterDto, Integer)
    */
   @Named("network:get")
   @EnterpriseEdition
   @GET
   @Consumes(VLANNetworkDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   @Fallback(NullOnNotFoundOr404.class)
   ListenableFuture<VLANNetworkDto> getNetwork(
         @EndpointLink("network") @BinderParam(BindToPath.class) DatacenterDto datacenter,
         @BinderParam(AppendToPath.class) Integer networkId);

   /**
    * @see InfrastructureApi#createNetwork(DatacenterDto, VLANNetworkDto)
    */
   @Named("network:create")
   @EnterpriseEdition
   @POST
   @Produces(VLANNetworkDto.BASE_MEDIA_TYPE)
   @Consumes(VLANNetworkDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<VLANNetworkDto> createNetwork(
         @EndpointLink("network") @BinderParam(BindToPath.class) DatacenterDto datacenter,
         @BinderParam(BindToXMLPayload.class) VLANNetworkDto network);

   /**
    * @see InfrastructureApi#updateNetwork(VLANNetworkDto)
    */
   @Named("network:update")
   @EnterpriseEdition
   @PUT
   @Produces(VLANNetworkDto.BASE_MEDIA_TYPE)
   @Consumes(VLANNetworkDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<VLANNetworkDto> updateNetwork(
         @EndpointLink("edit") @BinderParam(BindToXMLPayloadAndPath.class) VLANNetworkDto network);

   /**
    * @see InfrastructureApi#deleteNetwork(VLANNetworkDto)
    */
   @Named("network:delete")
   @EnterpriseEdition
   @DELETE
   ListenableFuture<Void> deleteNetwork(@EndpointLink("edit") @BinderParam(BindToPath.class) VLANNetworkDto network);

   /**
    * @see InfrastructureApi#checkTagAvailability(DatacenterDto, Integer)
    */
   @Named("network:checktag")
   @EnterpriseEdition
   @GET
   @Path("/datacenters/{datacenter}/network/action/checkavailability")
   @Consumes(VlanTagAvailabilityDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<VlanTagAvailabilityDto> checkTagAvailability(
         @PathParam("datacenter") @ParamParser(ParseDatacenterId.class) DatacenterDto datacenter,
         @QueryParam("tag") Integer tag);

   /*********************** Public Network IPs ***********************/

   /**
    * @see InfrastructureApi#listPublicIps(VLANNetworkDto)
    */
   @Named("publicnetwork:listips")
   @GET
   @Consumes(PublicIpsDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<PublicIpsDto> listPublicIps(
         @EndpointLink("ips") @BinderParam(BindToPath.class) VLANNetworkDto network);

   /**
    * @see InfrastructureApi#listPublicIps(VLANNetworkDto, IpOptions)
    */
   @Named("publicnetwork:listips")
   @GET
   @Consumes(PublicIpsDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<PublicIpsDto> listPublicIps(
         @EndpointLink("ips") @BinderParam(BindToPath.class) VLANNetworkDto network, IpOptions options);

   /**
    * @see InfrastructureApi#getPublicIp(VLANNetworkDto, Integer)
    */
   @Named("publicnetwork:getip")
   @GET
   @Consumes(PublicIpDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<PublicIpDto> getPublicIp(
         @EndpointLink("ips") @BinderParam(BindToPath.class) VLANNetworkDto network,
         @BinderParam(AppendToPath.class) Integer ipId);

   /**
    * @see InfrastructureApi#listExternalIps(VLANNetworkDto)
    */
   @Named("externalnetwork:listips")
   @GET
   @Consumes(ExternalIpsDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<ExternalIpsDto> listExternalIps(
         @EndpointLink("ips") @BinderParam(BindToPath.class) VLANNetworkDto network);

   /**
    * @see InfrastructureApi#listExternalIps(VLANNetworkDto, IpOptions)
    */
   @Named("externalnetwork:listips")
   @GET
   @Consumes(ExternalIpsDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<ExternalIpsDto> listExternalIps(
         @EndpointLink("ips") @BinderParam(BindToPath.class) VLANNetworkDto network, IpOptions options);

   /**
    * @see InfrastructureApi#getExternalIp(VLANNetworkDto, Integer)
    */
   @Named("externalnetwork:getip")
   @GET
   @Consumes(ExternalIpDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<ExternalIpDto> getExternalIp(
         @EndpointLink("ips") @BinderParam(BindToPath.class) VLANNetworkDto network,
         @BinderParam(AppendToPath.class) Integer ipId);

   /**
    * @see InfrastructureApi#listUnmanagedIps(VLANNetworkDto)
    */
   @Named("unmanagednetwork:listips")
   @GET
   @Consumes(UnmanagedIpsDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<UnmanagedIpsDto> listUnmanagedIps(
         @EndpointLink("ips") @BinderParam(BindToPath.class) VLANNetworkDto network);

   /**
    * @see InfrastructureApi#listUnmanagedIps(VLANNetworkDto, IpOptions)
    */
   @Named("unmanagednetwork:listips")
   @GET
   @Consumes(UnmanagedIpsDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<UnmanagedIpsDto> listUnmanagedIps(
         @EndpointLink("ips") @BinderParam(BindToPath.class) VLANNetworkDto network, IpOptions options);

   /**
    * @see InfrastructureApi#getUnmanagedIp(VLANNetworkDto, Integer)
    */
   @Named("unmanagednetwork:getip")
   @GET
   @Consumes(UnmanagedIpDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<UnmanagedIpDto> getUnmanagedIp(
         @EndpointLink("ips") @BinderParam(BindToPath.class) VLANNetworkDto network,
         @BinderParam(AppendToPath.class) Integer ipId);

   /***************************** NETWORK SERVICE TYPES *******************************************/

   /**
    * @see InfrastructureApi#listNetworkServiceTypes(DatacenterDto)
    */
   @Named("networkservicetype:list")
   @GET
   @Consumes(NetworkServiceTypesDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<NetworkServiceTypesDto> listNetworkServiceTypes(
         @EndpointLink("networkservicetypes") @BinderParam(BindToPath.class) DatacenterDto datacenter);

   /**
    * @see InfrastructureApi#createNetworkServiceType(DatacenterDto,
    *      NetworkServiceTypeDto)
    */
   @Named("networkservicetype:create")
   @POST
   @Produces(NetworkServiceTypeDto.BASE_MEDIA_TYPE)
   @Consumes(NetworkServiceTypeDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<NetworkServiceTypeDto> createNetworkServiceType(
         @EndpointLink("networkservicetypes") @BinderParam(BindToPath.class) DatacenterDto datacenter,
         @BinderParam(BindToXMLPayload.class) NetworkServiceTypeDto nst);

   /**
    * @see InfrastructureApi#getNetworkServiceType(DatacenterDto, Integer)
    */
   @Named("networkservicetype:get")
   @GET
   @Consumes(NetworkServiceTypeDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   @Fallback(NullOnNotFoundOr404.class)
   ListenableFuture<NetworkServiceTypeDto> getNetworkServiceType(
         @EndpointLink("networkservicetypes") @BinderParam(BindToPath.class) DatacenterDto datacenter,
         @BinderParam(AppendToPath.class) Integer nstId);

   /**
    * Update the value of a {@link NetworkServiceTypeDto}
    * 
    * @param nstDto
    *           the instance to update with the new values.
    * @return the updated entity.
    */
   @Named("networkservicetype:update")
   @PUT
   @Produces(NetworkServiceTypeDto.BASE_MEDIA_TYPE)
   @Consumes(NetworkServiceTypeDto.BASE_MEDIA_TYPE)
   @JAXBResponseParser
   ListenableFuture<NetworkServiceTypeDto> updateNetworkServiceType(
         @EndpointLink("edit") @BinderParam(BindToXMLPayloadAndPath.class) NetworkServiceTypeDto nstDto);

   /**
    * Remove a {@link NetworkServiceTypeDto} entity.
    * 
    * @param nstDto
    *           the entity to delete
    */
   @Named("networkservicetype:delete")
   @DELETE
   ListenableFuture<Void> deleteNetworkServiceType(
         @EndpointLink("edit") @BinderParam(BindToPath.class) NetworkServiceTypeDto nstDto);
}
